package org.bitbucket.bradleysmithllc.etlunit.feature.debug;

import com.google.inject.name.Named;
import org.bitbucket.bradleysmithllc.etlunit.*;
import org.bitbucket.bradleysmithllc.etlunit.feature.results.ResultsFeatureModule;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestClass;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;
import org.bitbucket.bradleysmithllc.etlunit.util.StringUtils;

import javax.inject.Inject;
import java.util.List;

public class ConsoleStatusReporter implements StatusReporter
{
	private Log applicationLog;
	private Log userLog;

	private ETLTestClass currentClass;

	private ResultsFeatureModule resultsFeatureModule;

	@Inject
	public void setResultsFeature(ResultsFeatureModule feature)
	{
		resultsFeatureModule = feature;
	}

	@Inject
	public void setApplicationLog(@Named("applicationLog") Log log)
	{
		this.applicationLog = log;
		log.info("Console Status Reporter online");
	}

	@Inject
	public void setUserLog(@Named("userLog") Log log)
	{
		this.userLog = log;
	}

	@Override
	public void scanStarted()
	{
		applicationLog.info("scanStarted");
	}

	@Override
	public void scanCompleted()
	{
		applicationLog.info("scanCompleted");
	}

	@Override
	public void testsStarted(int numTestsSelected)
	{
		applicationLog.info("testsStarted(" + numTestsSelected + ")");
	}

	@Override
	public void testClassAccepted(ETLTestClass method)
	{
		applicationLog.info("testClassAccepted(" + method.getQualifiedName() + ")");
	}

	@Override
	public void testMethodAccepted(ETLTestMethod method)
	{
		applicationLog.info("testMethodAccepted(" + method.getName() + ")");
	}

	@Override
	public void testBeginning(ETLTestMethod method)
	{
		if (currentClass == null || currentClass != method.getTestClass())
		{
			String
					message =
					("class "
							+ method.getTestClass().getQualifiedName()
							+ "   --------------------------------------------------------------------------------").substring(0, 82);

			userLog.info(message);
			currentClass = method.getTestClass();
		}

		TestResults testClassResults = resultsFeatureModule.getTestClassResults();
		TestResultMetrics metrics = testClassResults.getMetrics();

		String
				text =
				String.valueOf(metrics.getNumberOfTestsRun() + 1)
						+ '/'
						+ String.valueOf(testClassResults.getNumTestsSelected())
						+ "           ";

		userLog.info(text.substring(0, 11) + "." + method.getName());
	}

	@Override
	public void testCompleted(ETLTestMethod method, CompletionStatus status)
	{
		applicationLog.info("testCompleted(" + method.getQualifiedName() + ", " + status + ")");

		String state = "  Passed";

		List<TestAssertionFailure> assertionFailures = status.getAssertionFailures();

		switch (status.getTestResult())
		{
			case success:
				state = "  Passed";
				break;
			case failure:
				StringBuilder builder = new StringBuilder("  Failed");

				if (assertionFailures.size() != 0)
				{
					for (TestAssertionFailure failure : assertionFailures)
					{
						userLog.info("      " + failure.getFailureId());
					}
				}

				state = builder.toString();

				break;
			case warning:
				state = "  *Passed*";
				break;
			case error:
				TestExecutionError error = status.getError();
				if (error != null)
				{
					applicationLog.severe("Exception during testing", error);
				}

				userLog.info("      " + (error == null ? "" : (error.getErrorId())));

				state = "  Caused an error";
				break;
		}

		TestResults testClassResults = resultsFeatureModule.getTestClassResults();
		TestResultMetrics metrics = testClassResults.getMetrics();

		// print the pretty format
		StringBuilder sb = new StringBuilder('[')
				.append(" P[")
				.append(metrics.getNumberOfTestsPassed())
				.append("] F[")
				.append(metrics.getNumberOfAssertionFailures())
				.append("] E[")
				.append(metrics.getNumberOfErrors())
				.append("] W[")
				.append(metrics.getNumberOfWarnings())
				.append("]");

		String text = StringUtils.wrapToLength(sb.toString(), 80, ">>", "<<\n", StringUtils.RIGHT_JUSTIFIED);

		text = state + text.substring(state.length());

		userLog.info(text);
		userLog.info("");
	}

	@Override
	public void testsCompleted()
	{
		applicationLog.info("testsCompleted()");
	}
}